
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.012.
use strict;
use warnings;

use Module::Build 0.28;
use lib qw{inc}; use Module::Build::Parse::Yapp;

my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28",
    "Module::Build::Parse::Yapp" => "v0.1.2"
  },
  "configure_requires" => {
    "Module::Build" => "0.28",
    "Module::Build::Parse::Yapp" => "v0.1.2"
  },
  "dist_abstract" => "OpenSMILES format reader and writer",
  "dist_author" => [
    "Andrius Merkys <merkys\@cpan.org>"
  ],
  "dist_name" => "Chemistry-OpenSMILES",
  "dist_version" => "0.12.2",
  "license" => "bsd",
  "module_name" => "Chemistry::OpenSMILES",
  "recursive_test_files" => 1,
  "requires" => {
    "Chemistry::Elements" => 0,
    "Graph" => "0.97",
    "List::Util" => "1.45",
    "Parse::Yapp" => 0,
    "Set::Object" => 0,
    "perl" => "5.010000"
  },
  "test_requires" => {
    "Algorithm::Combinatorics" => 0,
    "Data::Dumper" => 0,
    "Test::More" => 0
  }
);


my %fallback_build_requires = (
  "Algorithm::Combinatorics" => 0,
  "Data::Dumper" => 0,
  "Module::Build" => "0.28",
  "Module::Build::Parse::Yapp" => "v0.1.2",
  "Test::More" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build::Parse::Yapp->new(%module_build_args);
$build->add_build_element($_) for qw(yp);

$build->create_build_script;
